/**
 * Copyright (c) 2015-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry.cli.command;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.cli.api.command.AbstractCommand;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.registry.client.api.exception.UnexistingTopologyException;

/**
 * This command delete the given topology registered in the Petals Registry cluster.
 * 
 * @author Christophe DENEUX - Linagora
 */
public class DeleteTopology extends AbstractCommand {

    public static final String NAME = "delete-topology";

    protected static final String TOPOLOGY_SHORT_OPTION = "t";

    protected static final String TOPOLOGY_LONG_OPTION = "topology";

    private static final Option TOPOLOGY_OPTION = Option.builder(TOPOLOGY_SHORT_OPTION).required()
            .longOpt(TOPOLOGY_LONG_OPTION)
            .desc("<topology> is the name of the topology to remove from the Petals Registry Overlay cluster.")
            .numberOfArgs(1).argName("topology").build();

    private final DefaultParser clParser;

    public DeleteTopology() {
        super(NAME);
        this.clParser = new DefaultParser();
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("Delete the given topology");
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
    }

    @Override
    public void doExecute(final String[] args) throws CommandException {
        try {
            final CommandLine cli = this.clParser.parse(getOptions(), args);
            if (cli.getArgs().length > 0) {
                throw new CommandBadArgumentNumberException(this);
            }

            Connect.PETALS_REGISTRY_CLIENT.clean(cli.getOptionValue(TOPOLOGY_SHORT_OPTION));

        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final UnexistingTopologyException e) {
            throw new CommandException(this, e);
        }
    }

    @Override
    public Options createOptions() {
        final Options options = new Options();
        options.addOption(TOPOLOGY_OPTION);
        return options;
    }
}
