/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry.cli.connection;

import java.net.UnknownHostException;

import org.ow2.petals.cli.api.connection.AuthenticatedConnectionParameters;
import org.ow2.petals.cli.api.connection.exception.HostMissingException;
import org.ow2.petals.cli.api.connection.exception.InvalidPortException;
import org.ow2.petals.cli.api.connection.exception.PasswordMissingException;
import org.ow2.petals.cli.api.connection.exception.UsernameMissingException;

public class AuthenticatedConnectionParametersImpl extends AuthenticatedConnectionParameters {
    
    private final String group;
    
    private final String password;

    /**
     * 
     * @param host
     * @param port
     * @param group
     *            Username. Can not be <code>null</code>
     * @param password
     *            Password. Can not be <code>null</code>
     * @throws UnknownHostException
     *             The host name can not be determined, or the IP address is invalid.
     * @throws HostMissingException
     *             The host is <code>null</code>
     * @throws InvalidPortException
     *             The host is invalid
     * @throws UsernameMissingException
     *             The user name is <code>null</code>
     * @throws PasswordMissingException
     *             The password is <code>null</code>
     */
    public AuthenticatedConnectionParametersImpl(final String host, final int port, final String group,
            final String password) throws UnknownHostException, HostMissingException,
            InvalidPortException, UsernameMissingException, PasswordMissingException {
        super(host, port);

        if (group == null) {
            throw new UsernameMissingException();
        }
        if (password == null) {
            throw new PasswordMissingException();
        }

        this.group = group;
        this.password = password;
    }

    public String getGroupName() {
        return this.group;
    }

    public String getPassword() {
        return this.password;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = super.hashCode();

        assert this.group != null : "username is not null, see constructor";
        assert this.password != null : "password is not null, see constructor";

        result = prime * result + (this.password.hashCode());
        result = prime * result + ((this.group == null) ? 0 : this.group.hashCode());
        return result;
    }

    @Override
    public boolean equals(final Object obj) {
        if (this == obj)
            return true;
        if (!super.equals(obj))
            return false;
        if (getClass() != obj.getClass())
            return false;

        assert this.group != null : "username is not null, see constructor";
        assert this.password != null : "password is not null, see constructor";

        final AuthenticatedConnectionParametersImpl other = (AuthenticatedConnectionParametersImpl) obj;
        if (!this.password.equals(other.password)) {
            return false;
        }
        if (!this.group.equals(other.group)) {
            return false;
        }
        return true;
    }
}
