/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry.cli.command;

import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.junit.Assert;
import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.api.connection.exception.HostMissingException;
import org.ow2.petals.cli.api.connection.exception.InvalidPortException;
import org.ow2.petals.cli.api.connection.exception.PasswordMissingException;
import org.ow2.petals.cli.api.connection.exception.UsernameMissingException;
import org.ow2.petals.cli.api.pref.PreferenceFileException;
import org.ow2.petals.cli.api.pref.Preferences;
import org.ow2.petals.cli.api.pref.exception.MissingDefaultPreferenceFileException;
import org.ow2.petals.cli.api.pref.exception.PreferenceFileNotFoundException;
import org.ow2.petals.cli.base.junit.AbstractConsoleInOutTest;
import org.ow2.petals.registry.cli.Main;
import org.ow2.petals.registry.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.registry.cli.pref.PreferencesImpl;

/**
 * Abstract class with useful method for command test.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class AbstractArtifactCommandTest extends AbstractConsoleInOutTest {

    public static final String PETALS_REG_PREF_FILE_ENV_VAR = "PETALS_REG_CLI_PREFS";

    protected void setUpPrefFile(final Properties preferenceProperties) throws Exception {
        super.setUpPrefFile(PETALS_REG_PREF_FILE_ENV_VAR, preferenceProperties);
    }

    protected void runCli(final boolean endInError, final String... args) {
        this.runCli(endInError, new PreferencesImpl(), args);
    }

    protected void runCli(final String... args) {
        this.runCli(false, args);
    }

    protected void runCli(final Preferences prefs, final String... args) {
        runCli(false, prefs, args);
    }

    protected void runCli(final boolean endInError, final Preferences prefs, final String... args) {
        final Runnable petalsCli = new Runnable() {

            @Override
            public void run() {

                final int exitCode = new Main().run(args, prefs, null);

                Assert.assertEquals("invalid exit code", endInError ? 1 : 0, exitCode);

            }
        };

        this.runCli(petalsCli);
    }

    /**
     * Create a basic preference configuration
     * 
     * @param host
     * @param port
     * @param group
     * @param password
     * @return A basic preference configuration
     */
    protected Preferences createBasicPreferences(final String host, final int port, final String groupname,
            final String password) {

        try {
            final ConnectionParameters connectionParameters = new AuthenticatedConnectionParametersImpl(host, port,
                    groupname, password);
            return new Preferences() {

                @Override
                public void intializePreferenceParameters() throws MissingDefaultPreferenceFileException,
                        PreferenceFileNotFoundException, PreferenceFileException {
                    // NOP
                }

                @Override
                public Map<String, ConnectionParameters> getPreferenceConnectionParameters() {
                    final Map<String, ConnectionParameters> connectionParametersMap = new HashMap<String, ConnectionParameters>();
                    connectionParametersMap.put("default", connectionParameters);
                    return connectionParametersMap;
                }

                @Override
                public ConnectionParameters getDefaultConnectionParameters() {
                    return connectionParameters;
                }
            };
        } catch (final UnknownHostException e) {
            // NOP: This exception has not to occur
            return null;
        } catch (final HostMissingException e) {
            // NOP: This exception has not to occur
            return null;
        } catch (final InvalidPortException e) {
            // NOP: This exception has not to occur
            return null;
        } catch (final UsernameMissingException e) {
            // NOP: This exception has not to occur
            return null;
        } catch (final PasswordMissingException e) {
            // NOP: This exception has not to occur
            return null;
        }
    }
}
