/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry.cli.command;

import static org.hamcrest.Matchers.containsString;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.io.IOException;

import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.base.junit.shell.DummyShell;
import org.ow2.petals.cli.base.junit.shell.PetalsInteractiveCliUtils;
import org.ow2.petals.cli.shell.ShellFactory;
import org.ow2.petals.registry.cli.Main;
import org.ow2.petals.registry.client.mock.junit.PetalsRegistryClientApi;
import org.ow2.petals.registry_overlay.api.Constants;

/**
 * Unit test of {@link Disconnect}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DisconnectTest extends AbstractArtifactCommandTest {

    /**
     * We use the mock implementation of Petals Registry Client API
     */
    @Rule
    public final PetalsRegistryClientApi prca = new PetalsRegistryClientApi();

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() throws CommandException {

        final DummyShell dummyShell = new DummyShell();
        final Disconnect command = new Disconnect();
        command.setShell(dummyShell);
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage",
                usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage",
                usage.contains(command.getName()));
    }

    /**
     * Test of the command "disconnect", where:
     * <ul>
     * <li>a connection is previously established,</li>
     * <li>the command 'disconnect' is executed before to exit.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>the connection is established before to execute 'disconnect',</li>
     * </li>the connection is released after 'disconnect', and before 'exit'.</li>
     * </ul>
     * </p>
     */
    @Test()
    public void testExecute_1() throws IOException, InterruptedException {

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            runCli(createBasicPreferences("localhost", Constants.DEFAULT_PORT,
                    Constants.DEFAULT_CREDENTIALS_GROUP, Constants.DEFAULT_CREDENTIALS_PASSWORD), "-" + ShellFactory.CONSOLE_SHORT_OPTION);
            
            // Wait that Petals CLI starts
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            // Connect
            systemIn.writeLine("connect");

            // Wait the execution of the command
            checkOut(containsString("Would you like to connect to " + Constants.DEFAULT_CREDENTIALS_GROUP
                    + ":*****@localhost:" + String.valueOf(Constants.DEFAULT_PORT) + "? (y/n)"));

            // Confirmation
            systemIn.writeLine("y");

            // Wait the execution of the confirmation
            checkOut(containsString(Main.PROMPT_BASE + "@localhost:" + String.valueOf(Constants.DEFAULT_PORT)));

            assertNotNull("Connection not established", Connect.PETALS_REGISTRY_CLIENT);

            // Enter the command
            systemIn.writeLine("disconnect");

            // Wait the execution of the command
            checkOut(containsString(Main.PROMPT_BASE + ">"));
            assertNull("Connection not disconnected", Connect.PETALS_REGISTRY_CLIENT);

            // Enter the command
            systemIn.writeLine("exit");

            waitThreadOrFail();
        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

}
